<?php

/**
 * Carerix Wordpress Plugin
 *
 *
 * @category Interfacer
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Jul 25, 2013 11:05:58 AM
 * @version $Rev$
 */
class MockIclTranslationManagement {

  function post_md5() {
    return uniqid();
  }

}

/**
 * Interfacer class
 * Used to encapsulate Wordpress's functions
 */
class WP_Interfacer {

  private static $_instance = null;
  private $db = null;
  private $prefix = 'carerix';
    public static $langMap = array(
        'en' => 'English',
        'nl' => 'Dutch',
        'de' => 'German',
        'fr' => 'French',
        'es' => 'Spanish',
        'hu' => 'Hungarian'
    );

  private function __construct($db) {
    if ($db === NULL) {
      throw new InvalidArgumentException(
      'Null database object provided to uninitialized WP_Interfacer');
    }
    $this->db = $db;
  }

  private function __clone() {
    
  }

  /**
   * 
   * @param wpdb $db
   * @return WP_Interfacer
   */
  public static function getInstance($db = NULL) {
    if (!is_object(self::$_instance)) {
      self::$_instance = new WP_Interfacer($db);
    }
    
    return self::$_instance;
  }

// <editor-fold defaultstate="collapsed" desc="Wordpress option encapsulation">

  /**
   * Adds or updates an option
   * 
   * @param string $key 
   * @param mixed $value
   * @return bool True on success, False on failure
   */
  public function setOption($key, $value) {
    $key = $this->prefixed($key);
    return update_option($key, $value);
  }

  /**
   * 
   * @param string $key Name of the option to delete
   * @return boolean True on success, False on failure
   */
  public function deleteOption($key) {
    $key = $this->prefixed($key);
    return delete_option($key);
  }

  /**
   * 
   * @param string $key Name of the option to retrieve
   * @param (mixed)(optional) $default Default value to return, if key not in the database, default FALSE
   * @return (mixed) If the option does not exist, returns FALSE
   */
  public function getOption($key, $default = FALSE) {
    $key = $this->prefixed($key);
    return get_option($key, $default);
  }

// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="Wordpress hooks encapsulation">

  /**
   * 
   * @param (mixed) $callable String or array of ClassName and FunctionName
   */
  public function on_init($callable) {
    add_action('init', $callable);
  }

  public function on_activate($callable) {
    register_activation_hook(CARERIX_PLUGIN_FILE, $callable);
  }

  public function on_before_admin_bar_render($callable) {
    add_action('wp_before_admin_bar_render', $callable);
  }

  // @todo [TO IMPLEMENT]
  public function on_deactivate() {
    // register_deactivation_hook(__FILE__, array($this, 'deactivate'));
  }

  public function on_admin_menu($callable) {
    add_action('admin_menu', $callable);
  }

  public function on_user_profile_display($callable) {
    add_action('edit_user_profile', $callable);
    add_action('show_user_profile', $callable);
  }

  public function on_user_profile_update($callable) {
    add_action('personal_options_update', $callable);
    add_action('edit_user_profile_update', $callable);
  }

  public function on_help($callable) {
    add_filter('contextual_help', $callable, 10, 3);
  }

  public function on_frontend_content($callable) {
    add_filter('the_content', $callable, 1);
  }

  public function on_posts_columns_heads($callable) {
    add_filter('manage_posts_columns', $callable);
  }

  public function on_posts_columns_contents($callable) {
    add_filter('manage_posts_custom_column', $callable, 10, 2);
  }

  public function frontend_displayed_single() {
    return is_single();
  }

  public function frontend_displayed_page() {
    return is_page();
  }

  public function on_shortcode($name, $callable) {
    add_shortcode($name, $callable);
  }

  public function add_cron_schedule($callable) {
    add_filter('cron_schedules', $callable);
  }

  public function schedule_next_cron($interval, $hook_name) {
    if (!wp_next_scheduled($hook_name)) {
      $nexttime = strtotime("+5 minutes");
      $start_time = mktime(date('H', $nexttime), 0, 0, date('m', $nexttime), date('d', $nexttime), date('Y', $nexttime));
      wp_schedule_event($start_time, $interval, $hook_name);
    }
  }

  public function on_action($action_name, $callable) {
    add_action($action_name, $callable);
  }

  public function register_script_asset($script_relative_path) {
    $qname = $this->only_name($script_relative_path);
    $script_url = plugins_url('/assets/scripts/' . $script_relative_path, CARERIX_PLUGIN_FILE);
    wp_enqueue_script($qname, $script_url);
  }

  public function register_style_asset($style_relative_path) {
    $qname = $this->only_name($style_relative_path);
    $style_url = plugins_url('/assets/styles/' . $style_relative_path, CARERIX_PLUGIN_FILE);
    wp_register_style($qname, $style_url, array(), Carerix::VERSION);
    wp_enqueue_style($qname);
  }

// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="Wordpress user encapsulation">

  public function user_is_authenticated() {
    return is_user_logged_in();
  }

  public function user_is_admin() {
    return is_admin();
  }

  public function is_frontend() {
    if (stripos($_SERVER['REQUEST_URI'], 'wp-admin') === FALSE && stripos($_SERVER['REQUEST_URI'], 'wp-login') === FALSE && stripos($_SERVER['REQUEST_URI'], 'wp-register') === FALSE
    ) {
      return TRUE;
    }

    return FALSE;
  }

  public function user_can($user_id, $action) {
    return current_user_can($action, $user_id);
  }

  public function user_meta_set($user_id, $meta_key, $meta_value) {
    update_usermeta($user_id, $meta_key, $meta_value);
  }

  public function user_meta_get($user_id) {
    return get_user_meta($user_id);
  }

  public function user_current_id() {
    return get_current_user_id();
  }

  public function user_current_name() {
    $current_user = wp_get_current_user();
    return $current_user->display_name;
  }

  public function user_current_time() {
    return current_time('mysql');
  }

  public function user_insert($user_login, $user_pass) {
    $result = wp_insert_user(array(
      'user_pass' => $user_pass,
      'user_login' => $user_login,
    ));
    if (!is_wp_error($result))
      return $result;

    return false;
  }

  public function user_id_by_login($login) {
    $users_table = $this->db->users;

    $query = $this->db->prepare("SELECT id FROM $users_table WHERE user_login = %s", $login);
    $users = $this->db->get_results($query);

    if (count($users) == 0 || !isset($users[0]->id) || $users[0]->id === NULL) {
      return FALSE;
    }

    return $users[0]->id;
  }

  public function user_by_id($id) {
    $users_table = $this->db->users;
    
    $query = $this->db->prepare("SELECT * FROM $users_table WHERE id = %d", $id);
    $users = $this->db->get_results($query);

    if (count($users) == 0 || !isset($users[0]->ID) || $users[0]->ID === NULL) {
      return FALSE;
    }

    return $users[0];
  }

// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="Wordpress menu management encapsulation">

  /**
   * 
   * @param WP_Page $page
   */
  function add_page($page) {

    if (!is_a($page, 'WP_Page')) {
      throw new DomainException('The provided object is not of type WP_Page');
    }

    add_menu_page($page->getPageTitle(), $page->getMenuTitle(), $page->getCapability(), $page->getMenuSlug(), array($page, 'page'));
  }

  /**
   * 
   * @param WP_Page $parent
   * @param WP_Page $child
   */
  function add_sub_page($parent, $child) {
    if (!is_a($parent, 'WP_Page')) {
      throw new DomainException('The parent page is not of type WP_Page');
    }

    if (!is_a($child, 'WP_Page')) {
      throw new DomainException('The child page is not of type WP_Page');
    }

    add_submenu_page($parent->getMenuSlug(), $child->getPageTitle(), $child->getMenuTitle(), $child->getCapability(), $child->getMenuSlug(), array($child, 'page'));
  }

// </editor-fold>	
// <editor-fold defaultstate="collapsed" desc="Wordpress database management encapsulation">	

  function db() {
    return $this->db;
  }

  function db_get_prefix() {
    return $this->db->prefix;
  }

  function db_get_posts_table_name() {
    return $this->db->prefix . 'posts';
  }

  function db_get_terms_table_name() {
    return $this->db->prefix . 'terms';
  }

  function db_get_postmeta_name() {
    return $this->db->prefix . 'postmeta';
  }

  function db_get_by_id($table, $id) {
    $query = "SELECT * FROM $table WHERE `id` = %d";
    $prepared_query = $this->db->prepare($query, $id);

    $results = $this->db->get_results($prepared_query, ARRAY_A);

    if (count($results) !== 1) {
      return FALSE;
    }

    return $results[0];
  }

  function db_get_by_column($table, $column_name, $column_value) {
    $query = "SELECT * FROM $table WHERE `$column_name` = %s";
    $prepared_query = $this->db->prepare($query, $column_value);
    $results = $this->db->get_results($prepared_query, ARRAY_A);

    return $results;
  }

  function db_get_custom($query) {
    $results = $this->db->get_results($query, ARRAY_A);
    return $results;
  }

  function db_get_all($table) {
    $query = "SELECT * FROM $table";
    $results = $this->db->get_results($query, ARRAY_A);
    return $results;
  }

  function db_save($table, $data) {
    $formatted = $this->get_formats_and_data($data);
    $formatted_data = $formatted['data'];

    $formats = $formatted['formats'];

    $this->db->insert($table, $formatted_data, $formats);

    return $this->db->insert_id;
  }

  function db_last_query() {
    echo '<textarea>';
    echo $this->db->last_query;
    echo '</textarea>';
  }

  function db_update($table, $id, $data) {
    $where = array('id' => $id);
    $where_format = array('%d');

    $formatted = $this->get_formats_and_data($data);
    $formatted_data = $formatted['data'];
    $data_formats = $formatted['formats'];

    return $this->db->update(
            $table, $formatted_data, $where, $data_formats, $where_format
    );
  }

  function db_delete($table, $id) {
    return $this->db->delete($table, array('id' => $id), array('%d'));
  }

  function db_delete_all($table, $where = false) {
    $query = "DELETE FROM $table";
    if ($where !== FALSE) {
      $query .= ' WHERE ' . $where;
    }
    return $this->db->query($query);
  }

  function db_query($query) {
    return $this->db->query($query);
  }

// </editor-fold>	
// <editor-fold defaultstate="collapsed" desc="Wordpress language">

  function language_current() {
    return get_bloginfo('language');
  }

  function language_major_current() {
    /*
      if(FALSE !== $settings = $this->_wpml_settings())
      {
      return $settings['default_language'];
      }
     */
    $current_language = $this->language_current();
    $_parts = explode('-', $current_language);
    if (count($_parts) > 1)
      return $_parts[0];
    else
      return $current_language;
  }
  
    function get_carerix_language() {
        $lang = '';
        $current_language = strtolower($this->language_major_current());
        
        if (isset(self::$langMap[$current_language])) {
            $lang = self::$langMap[$current_language];
        }
        
        return $lang;
    }

  function get_wpml_active_languages() {
    if (FALSE !== $settings = $this->_wpml_settings()) {
      return $settings['default_categories'];
    }
    return FALSE;
  }

  private function _wpml_settings() {
    $settings = FALSE;
    $active_plugins = (array) get_option('active_plugins', array());
    $wpmlIsActive = in_array('sitepress-multilingual-cms/sitepress.php', $active_plugins);
    if ($wpmlIsActive) {
      $settings = get_option('icl_sitepress_settings');
    }
    return $settings;
  }

  public function wpml_is_active() {
    $active_plugins = (array) get_option('active_plugins', array());
    $wpmlIsActiveStatus = in_array('sitepress-multilingual-cms/sitepress.php', $active_plugins);
    return $wpmlIsActiveStatus;
  }

  public function wpml_table_name() {
    return $this->db->prefix . 'icl_translations';
  }

  public function wpml_unregister_translation($element_id) {
    if (!$this->managed_by_carerix($element_id)) {
      return false;
    }


    $wpmlTable = $this->wpml_table_name();

    $mysqli = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
    if ($mysqli->connect_error) {
      
    } else {
      $result = $mysqli->query("show tables like '$wpmlTable'");

      if ($result->num_rows == 1) {
        $deleteSql = $this->db->prepare("DELETE FROM $wpmlTable WHERE `element_id` = %d", $element_id);
        $this->db->query($deleteSql);
      }
    }
  }
  
  /**
   * Returns translation for page
   * @param Number $page_id Page ID
   * @param String $language Language code
   * @return stdClass
   */
  public function wpml_get_page_translation ($page_id, $language) {
        $wpmlTable = $this->wpml_table_name();
        $selectSql = $this->db->prepare("SELECT * FROM $wpmlTable WHERE `language_code` = %s "
                . "AND trid IN (SELECT `trid` FROM $wpmlTable WHERE `element_id` = %d and `element_type` = 'post_page')", array($language, $page_id));
      
        $entries = $this->db->get_results($selectSql);
        
        if (count($entries)) {
            return $entries[0];
        } else {
            return false;
        }
  }
  
  public function wpml_register_translation($element_id, $trid, $type, $language) {
    $trid = $trid * 10;

    // [VP] GEM:31319  Taxonomy is not a post so we do not need this check
    if (stripos($type, 'tax_') === false) {
        if (!$this->managed_by_carerix($element_id)) {
          return false;
        }
    }

    $wpmlTable = $this->wpml_table_name();
    $selectSql = $this->db->prepare("SELECT * FROM $wpmlTable WHERE `element_id` = %d AND `element_type` = %s", array($element_id, $type));
    $existentEntries = $this->db->get_results($selectSql);

    if (count($existentEntries) > 0) {
            $translation_id = $existentEntries[0]->translation_id;

            $this->remove_ghost_translations($language, $trid, $translation_id);

            $updateSql = $this->db->prepare("UPDATE $wpmlTable SET `language_code` = %s, `trid` = %s WHERE `translation_id` = %d", array(
                $language,
                $trid,
                $translation_id
            ));
            $this->db->query($updateSql);
    } else {
       $this->db->insert($wpmlTable, array(
        'element_type' => $type,
        'element_id' => $element_id,
        'trid' => $trid,
        'language_code' => $language
       ), array(
        '%s',
        '%d',
        '%s',
        '%s'
       ));
    }

    $existentEntries = null;
  }

// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="Wordpress role management">

  /**
   * 
   * @param string $system_name
   * @param string $display_name
   */
  function add_role($system_name, $display_name) {
    add_role($this->prefixed($system_name), $display_name);
  }

  /**
   * 
   * @param string $system_name
   * @param array $capabilities
   */
  function set_role_capabilities($system_name, $capabilities = array()) {
    $role = & get_role($this->prefixed($system_name));
    foreach ($capabilities as $capability) {
      $role->add_cap($capability);
    }
  }

// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="Wordpress taxonomy management">

  function taxonomy_clear_cache($taxonomy) {
    //delete_option("{$taxonomy}_children");
    $terms_ids = get_terms($taxonomy, array(
      'hide_empty' => false,
      'fields' => 'ids',
    ));
    clean_term_cache($terms_ids, $taxonomy, true);
  }

  function taxonomy_all($taxonomy_type, $parent) {
    $options = array(
      'hierarchical' => true,
      'hide_empty' => false,
      'parent' => $parent,
    );

    return get_terms($taxonomy_type, $options);
  }

  function taxonomy_get($taxonomy_type, $term_id) {
    return get_term($term_id, $taxonomy_type, ARRAY_A);
  }

  function taxonomy_insert($taxonomy_type, $term_name, $options) {
    return wp_insert_term($term_name, $taxonomy_type, $options);
  }

  function taxonomy_update($taxonomy_type, $term_id, $options) {
    return wp_update_term($term_id, $taxonomy_type, $options);
  }

  function taxonomy_delete($taxonomy_type, $term_id) {
    return wp_delete_term($term_id, $taxonomy_type);
  }

  function taxonomy_get_by_name($value, $taxonomy) {
    return get_term_by('name', $value, $taxonomy, ARRAY_A);
  }

// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="Wordpress entity management">

  function add_entity_type($type_name, $options) {
    register_post_type($type_name, $options);
  }

  function add_entity_category_type($category_name, $entity_type_name, $options) {
    register_taxonomy($category_name, $entity_type_name, $options);
  }

  function entity_delete($id, $force_delete = true) {
    return wp_delete_post($id, $force_delete);
  }

  function entity_delete_by_author($author) {
    $user_id = $this->user_id_by_login($author);

    if (!$user_id)
      return;
    $table = $this->db->prefix . 'posts';
    $postmeta = $this->db->prefix . 'postmeta';
    $posts = $this->db_get_by_column($table, 'post_author', $user_id);

    foreach ($posts as $post) {
      $post_id = $post['ID'];

      $this->entity_delete($post_id, true);
      $where = $this->db->prepare("post_id = %d AND (meta_key = 'source' OR meta_key LIKE '%carerix%')", $post_id);

      $this->db_delete_all($postmeta, $where);
    }
  }

  /**
   * should be set after init or on init
   * @param type $callable
   */
  function on_entity_delete($callable) {
    if (current_user_can('delete_posts')) {
      add_action('delete_post', $callable, 10);
    }
  }

  function on_category_delete($callable) {
    add_action('delete_category', $callable, 10);
  }

  function entity_get($id) {
    return get_post($id, ARRAY_A);
  }

  function entity_insert($data) {
    remove_action('post_updated', 'wp_save_post_revision');
    $result = wp_insert_post($data);
    add_action('post_updated', 'wp_save_post_revision', 10, 1);
    return $result;
  }

  function entity_update($id, $data = array()) {
    $data['ID'] = $id;
    remove_action('post_updated', 'wp_save_post_revision');
    $result = wp_update_post($data);
    unset($data);
    add_action('post_updated', 'wp_save_post_revision', 10, 1);
    return $result;
  }

  function entity_set_property($id, $property_name, $property_value) {
    if ($this->wpml_is_active()) {
      global $iclTranslationManagement;
      if (null === $iclTranslationManagement) {
        $iclTranslationManagement = new MockIclTranslationManagement();
      }
    }

    update_post_meta($id, $property_name, $property_value);
  }

  function entity_get_properties($entity_id) {
    return get_post_meta($entity_id);
  }

  function current_frontend_entity_get_properties() {
    global $post;
    return $this->entity_get_properties($post->ID);
  }

  function entities_get_all($type) {
    $page = 0;
    $per_page = 10;
    $input_data = filter_input_array(INPUT_GET, FILTER_SANITIZE_STRING);
    
    if (isset($input_data['pagenr']) && is_numeric($input_data['pagenr'])) {
      $page = $per_page * (int)$input_data['pagenr'];
    }

    $args = array(
      'post_type' => $type,
      'posts_per_page' => $per_page,
      'orderby' => 'post_date',
      'order' => 'DESC',
      'offset' => $page
    );
    return get_posts($args);
  }

  function entity_set_categories($entity_id, $term_ids, $taxonomy) {
    foreach ($term_ids as &$id) {
      $id = (int) $id;
    }
    return wp_set_object_terms($entity_id, $term_ids, $taxonomy, false);
  }

  function entity_get_categories($entity_id, $taxonomy) {
    return wp_get_object_terms($entity_id, $taxonomy);
  }


	/**
	 * Searches for posts based on an alias pubID (which is not in this environment)
	 *
	 * @param int $pub_id
	 *
	 * @return array
	 * @since 2.20.1
	 */
	function entities_get_where_aliaspubid($pub_id) {
	  $posts = get_posts(array(
		  'post_type' => 'post',
		  'numberposts' => 1,
		  'tax_query' => array(
			  array(
				  'taxonomy' => WP_Constants::TAXONOMY_PUBIDLIST,
				  'field' => 'name',
				  'terms' => $pub_id, // Where taxonomy `name` equals pub_id
				  'include_children' => false
			  )
		  )
	  ));

	  return $posts;
  }

  function entities_get_where_property($property_key, $property_value, $exactMatch = true) {
    $postmeta_table = $this->db->postmeta;
    if ($exactMatch) {
      $sql = $this->db->prepare("SELECT post_id FROM $postmeta_table "
                                . "WHERE meta_key = %s AND meta_value = %s", array($property_key, $property_value));
    } else {
      $sql = $this->db->prepare("SELECT post_id FROM $postmeta_table "
                                . "WHERE meta_key = %s AND meta_value LIKE %s", array($property_key, '%'. $property_value . '%'));
    }
    $posts = $this->db->get_results($sql);

    return $posts;
  }
  
    function entities_delete_where_property($post_id) {
        $postmeta_table = $this->db->postmeta;
        $this->db->delete($postmeta_table, array('post_id' => $post_id), array('%d'));
    }

  function entity_link_category($entity_id, $categories, $language = false) {
        if (!is_array($categories)) {
            return;
        }
      
        $categories = array_map('trim', array_filter($categories));
        $lastCategoryId = 0;
        $ids = array();

        foreach ($categories as $category) {
            if ($language !== FALSE) {
                $category = $category . ' - ' . $language;
            }

            $term_pair = term_exists(htmlentities($category), 'category', $lastCategoryId);

            if ($term_pair == null) {
                $lastCategoryId = (int) wp_create_category($category, (int) $lastCategoryId);
	            update_term_meta($lastCategoryId, "managed_by", WP_Constants::META_KEY_PREFIX);
            } else {
                $lastCategoryId = (int) $term_pair['term_id'];
	            update_term_meta($lastCategoryId, "managed_by", WP_Constants::META_KEY_PREFIX); // force this for existing cats from previous CX plugin versions
            }

            if ($language !== FALSE) {
                $this->wpml_register_translation($lastCategoryId, $lastCategoryId * 100000, 'tax_category', $language);
            }

            array_push($ids, $lastCategoryId);
        }

        wp_set_post_terms($entity_id, $ids, 'category');
    }

    function entity_set_tags($entity_id, $tags = array()) {
    if (empty($tags))
      return;

    wp_set_post_terms($entity_id, $tags, 'post_tag');
  }

  function entity_set_taxonomy($taxonomy_name, $entity_id, $taxonomies = array()) {
    if (empty($taxonomies))
      return;

    wp_set_post_terms($entity_id, $taxonomies, $taxonomy_name);
  }

  function get_term_id_by_name($name) {
    $table = $this->db_get_terms_table_name();
    $query = $this->db->prepare("SELECT term_id FROM $table WHERE `name` = %s", $name);
    $result = $this->db_get_custom($query);
    if (count($result) > 0)
      return (int) $result[0]['term_id'];
    return 0;
  }

	/**
	 * @deprecated Unused?
	 *
	 * @param        $category_name
	 * @param        $parent_id
	 * @param string $description
	 *
	 * @return int|object|WP_Error
	 */
  function set_category_parent($category_name, $parent_id, $description = '') {
    $data = array(
      'cat_ID' => $this->get_term_id_by_name($category_name),
      'cat_name' => $category_name,
      'category_description' => $description,
      'category_parent' => $parent_id,
    );

    return wp_insert_category($data);
  }

	/**
	 * @deprecated  Unused?
	 *
	 * @param      $categoryName
	 * @param bool $parentCategoryId
	 *
	 * @return int
	 */
  function create_category($categoryName, $parentCategoryId = false) {
    if ($parentCategoryId === FALSE) {
      return (int) wp_create_category($categoryName);
    } else {
      return (int) wp_create_category($categoryName, $parentCategoryId);
    }
  }

// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="utilities">	
  public function prefixed($string) {
    return $this->prefix . '_' . $string;
  }

  public function get_plugin_prefix() {
    return $this->prefix;
  }

  private function get_formats_and_data($data) {
    $formats = array();
    foreach ($data as $column => &$value) {
      $type = gettype($value);
      switch ($type) {
        case 'boolean':
        case 'integer':
          array_push($formats, '%d');
          break;
        case 'array':
          $value = serialize($value);
        case 'string':
          array_push($formats, '%s');
          break;
        default:
          throw new DomainException('Provided value is not of a valid type');
          break;
      }
    }
    return array('formats' => $formats, 'data' => $data);
  }

  public function get_post_permalink($post_id = false, $extra_query = false) {
    if ($post_id === FALSE) {
      global $post;
      $post_id = $post->ID;
    }

    $permalink = get_permalink($post_id);

    if ($extra_query) {
      if (strlen(trim(get_option('permalink_structure'))) > 0) {
        $permalink .= '?' . $extra_query;
      } else {
        $permalink .= '&' . $extra_query;
      }
    }

    return $permalink;
  }

// </editor-fold>


  public function message($m) {
    /*
      echo '<pre>';
      echo $m;
      echo '</pre>';
     */
  }

  public function url_merger($url, $extra) {
    $new_url = '';
    $current_data = parse_url($url);
    if (isset($current_data['query'])) {
      $parts = explode('&', $current_data['query']);
      foreach ($parts as $part) {
        list($qname, $qvalue) = explode('=', $part);

        if (isset($extra[$qname])) {
          $existent = explode('|', $extra[$qname]);
          if (!in_array($qvalue, $existent)) {
            $extra[$qname] = implode('|', array($extra[$qname], $qvalue));
          }
        } else {
          $extra[$qname] = $qvalue;
        }
      }
    }

    $query = '';
    foreach ($extra as $key => $value) {
      $query .= $key . '=' . $value . '&';
    }
    $query = rtrim($query, '&');

    $current_data['query'] = $query;

    if (isset($current_data['scheme']))
      $new_url .= $current_data['scheme'] . '://';
    if (isset($current_data['host']))
      $new_url .= $current_data['host'];
    if (isset($current_data['path']))
      $new_url .= $current_data['path'];
    if (isset($current_data['query']))
      $new_url .= '?' . $current_data['query'];

    return $new_url;
  }

  public function url_medium_for_translation($current_url, $language = false) {
    $new_url = $current_url;

    $current_data = parse_url($current_url);

    $base_mediums = array();

    $toReplace = false;
    if (isset($current_data['query'])) {
      $parts = explode('&', $current_data['query']);

      foreach ($parts as $part) {
        list($pname, $pvalue) = explode('=', $part);
        if ($pname == 'medium') {
          $toReplace = $part;
          $mediums = explode('|', $pvalue);

          foreach ($mediums as $medium) {
            $base_medium_parts = explode('-', $medium);

            if (count($base_medium_parts) > 1) {
              $last = count($base_medium_parts) - 1;
              unset($base_medium_parts[$last]);
            }

            $base_medium = implode('-', $base_medium_parts);

            array_push($base_mediums, $base_medium);
          }
          break;
        }
      }
    }

    $base_mediums = array_unique($base_mediums);

    if ($language !== FALSE) {
      foreach ($base_mediums as &$base_medium) {
        $base_medium .= '-' . $language;
      }
    }

    if ($toReplace !== FALSE) {
      $replacer = 'medium=' . implode('|', $base_mediums);
      $new_url = str_replace($toReplace, $replacer, $new_url);
    }

    return $new_url;
  }

  private function only_name($path) {
    $_t = explode('.', basename($path));
    if (count($_t) > 1)
      return $_t[0];
    return basename($path);
  }

  public function email($to, $subject, $message, $headers = false) {
    add_filter('wp_mail_content_type', array($this, 'set_html_content_type'));
    if ($headers !== false) {
      wp_mail($to, $subject, $message, $headers);
    } else {
      wp_mail($to, $subject, $message);
    }
    remove_filter('wp_mail_content_type', array($this, 'set_html_content_type'));
  }

  public function set_html_content_type() {
    return 'text/html';
  }

  public function on_head_display($callable) {
    add_action('wp_head', $callable);
  }

  public function on_ajax($ajax_name, $ajax_callable) {
    add_action('wp_ajax_nopriv_' . $ajax_name, $ajax_callable);
  }

  public function on_ajax_priv($ajax_name, $ajax_callable) {
    add_action('wp_ajax_' . $ajax_name, $ajax_callable);
  }

  public function managed_by_carerix($post_id) {
    $properties = $this->entity_get_properties($post_id);

    if (!empty($properties['managed_by']) && isset($properties['managed_by'][0]) && $properties['managed_by'][0] == WP_Constants::META_KEY_PREFIX) {
      if (isset($properties['vacancyID']) && !isset($properties['guid'])) {
        return false;
      }
      return true;
    }
    return false;
  }

  public function get_posts_managed_by_carerix() {
    $posts = array();
    $postmetaTable = $this->db->postmeta;

    $selectSql = "SELECT `post_id` FROM {$postmetaTable} WHERE `meta_key` =  'managed_by' AND  `meta_value` =  '_carerix_'";

    $managedByCarerixPosts = $this->db->get_results($selectSql);
    $managed = array();
    
    foreach($managedByCarerixPosts as $post) {
      array_push($managed, $post->post_id);
    }
    
    if(count($managed) > 0) {
      $posts_ids = '(' . implode(',', $managed) . ')';
      $selectSql = "SELECT `meta_value`, `post_id` FROM {$postmetaTable} WHERE `meta_key` =  'guid' AND  `post_id` in $posts_ids";
      $posts_pubs = $this->db->get_results($selectSql);
      foreach($posts_pubs as $postpub) {
        $posts[$postpub->meta_value] = $postpub->post_id;
      }
    }
    
    return $posts;
  }

    /**
     * Try to find ghost entities amd remove
     * @param $language
     * @param $trid
     * @param $translation_id
     */
  public function remove_ghost_translations ($language, $trid, $translation_id) {
      $wpmlTable = $this->wpml_table_name();

      $ghostEntries = $this->db->get_results($this->db->prepare("SELECT * FROM $wpmlTable WHERE `language_code` = %s AND `trid` = %s AND `translation_id` != %d", array(
          $language,
          $trid,
          $translation_id
      )));
      if (count($ghostEntries) > 0) {
          $this->db->delete($wpmlTable, array('translation_id' => $ghostEntries[0]->translation_id), array('%d'));
      }
  }
}
